@@ -1,7 +1,7 @@
 /* $Id$ */
 
 /*
- * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
+ * Copyright (c) 2006 Nicholas Marriott <nicholas.marriott@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,6 +18,7 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 
 #include <errno.h>
@@ -571,10 +572,15 @@ makessl(struct server *srv, int fd, int verify, int timeout, char **cause)
 	int	 n, mode;
 
 	ctx = SSL_CTX_new(SSLv23_client_method());
-	if (srv->tls1)
-		SSL_CTX_set_options(ctx, SSL_OP_ALL);
-	else
-		SSL_CTX_set_options(ctx, SSL_OP_ALL | SSL_OP_NO_TLSv1);
+	SSL_CTX_set_options(ctx, SSL_OP_ALL);
+
+	/* Disable insecure SSL/TLS versions. */
+	if (!srv->insecure) {
+		SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv2);
+		SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv3);
+		SSL_CTX_set_options(ctx, SSL_OP_NO_TLSv1);
+	}
+
 	SSL_CTX_set_default_verify_paths(ctx);
 	SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
 
@@ -584,6 +590,11 @@ makessl(struct server *srv, int fd, int verify, int timeout, char **cause)
 		goto error;
 	}
 
+	if (SSL_set_tlsext_host_name(ssl, srv->host) != 1) {
+		*cause = sslerror("SSL_set_tlsext_host_name");
+		goto error;
+	}
+
 	if (SSL_set_fd(ssl, fd) != 1) {
 		*cause = sslerror("SSL_set_fd");
 		goto error;
@@ -689,3 +700,37 @@ connectio(
 	}
 	return (io_create(fd, ssl, eol));
 }
+
+struct io *
+connectunix(const char *dest, char **cause)
+{
+	struct sockaddr_un	addr;
+	int			s;
+
+	if (*dest != '/') {
+		*cause = xstrdup("connectunix failed: path must be absolute");
+		return (NULL);
+	}
+
+	if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
+		xasprintf(cause, "connectunix failed: socket: %s",
+		    strerror(errno));
+		return (NULL);
+	}
+
+	memset(&addr, 0, sizeof addr);
+	addr.sun_family = AF_UNIX;
+	if (strlcpy(addr.sun_path, dest, sizeof addr.sun_path) >=
+	    sizeof addr.sun_path) {
+		*cause = xstrdup("connectunix failed: path is to long");
+		return (NULL);
+	}
+	if (connect(s, (struct sockaddr *)&addr, sizeof addr) == -1) {
+		xasprintf(cause, "connectunix failed: connect: %s",
+		    strerror(errno));
+		return (NULL);
+	}
+
+	return (io_create(s, NULL, IO_CRLF));
+}
+
